from os import environ

from dotenv import load_dotenv

from asc.core.ABC.APIRequestor import APIRequestor
from asc.core.tools.marketplace.mercadopago.data import PayerCustomer
from asc.core.tools.marketplace.mercadopago.data import Payment
from asc.core.tools.marketplace.mercadopago.data import Customer
from asc.core.tools.marketplace.mercadopago.data import Card

#TODO: Implementar/Importar exceptions
from asc.exceptions import mercadopago as exceptions_mercadopago

load_dotenv()


class MercadoPago(APIRequestor):

    base_payload = {}
    base_api_url = environ.get('mercadopago__url_baseapi')
    base_headers = {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'User-Agent': 'ASC-Tools-marketplace',
        'Authorization': ['Bearer', environ.get('mercadopago__access_token')]
    }

    _data_payer: PayerCustomer = None

    def __init__(self):
        super().__init__()

    def __response(self, response, parseAs: str = 'json'):
        if parseAs == 'json':
            resp = response.json()

            print('RESPONSE=',resp)

            if 'error' in resp:
                error_cause = resp['cause'][0]

                if error_cause['code'] == '101':
                    raise exceptions_mercadopago \
                        .CustomerAlreadyExistsException()

                if error_cause['code'] == '120':
                    raise exceptions_mercadopago \
                        .CardDoesNotExistsException()

                if error_cause['code'] == 'E205':
                    raise exceptions_mercadopago \
                        .CardInvalidExpirationYearException()

                raise exceptions_mercadopago.PaymentException(
                    f"{resp['message']} - {error_cause['description']} {error_cause['code']}"
                )

            return resp

        return response.text

    def GET_payment_methods(self):
        """
        Consulte todas as formas de pagamento disponíveis e obtenha
        uma lista com os detalhes de cada uma delas e suas propriedades.
        :return:
        """
        return self.__response(
            self.GET('v1/payment_methods')
        )

    def GET_identification_types(self):
        """
        Veja todos os tipos de documentos disponíveis por país e obtenha
        uma lista com a identificação e detalhes de cada um deles.
        :return:
        """
        return self.__response(
            self.GET('v1/identification_types')
        )

    def POST_payment(self, request_data:Payment):
        """ Envie o pagamento ao Mercado Pago.
        """
        print("PAYLOAD=",request_data)
        request_resp = self.__response(
            self.POST('v1/payments', request_data._asdict())
        )

        return request_resp

    def POST_payment_pix(self, transaction_amount:int, payer:PayerCustomer) -> dict:
        """
        Envia pagamento pix.
        :param amount: valor da cobrança.
        :param payer: dados do pagador.
        :return:
        """
        request_data = Payment(
            payment_method_id='pix',
            transaction_amount=transaction_amount,
            payer=payer,
            callback_url=environ.get('mercadopago__url_callback')
        )

        return self.POST_payment(request_data._asdict())

    def POST_customer(self, **data) -> dict:
        """
        Registra cliente.
        :return:
        """
        return self.__response(
            self.POST(
                'v1/customers',
                Customer(**data)._asdict()
            )
        )

    def GET_customer_search(self, email:str) -> dict:
        """
        Encontre todas as informações do cliente
        através de filtros específicos.
        :param email: email do cliente a buscar.
        :return:
        """
        return self.__response(
            self.GET(
                'v1/customers/search',
                {'email':email}
            )
        )

    def GET_customer(self, id:str) -> dict:
        """
        Veja todas as informações de um cliente
        a partir da sua identificação.
        :param id: id do cliente a buscar.
        :return:
        """
        return self.__response(
            self.GET(
                f'v1/customers/{id}'
            )
        )

    def POST_card(self, card:Card) -> dict:
        """
        Obtém token para um cartão.
        :return:
        """
        card = card._asdict()
        print('CARD',card)
        return self.__response(
            self.POST(
                f'v1/card_tokens',
                card
            )
        )

    def POST_customer_card(self, id_customer:str, token:str) -> dict:
        """
        Atrela cartão a um cliente.
        :return:
        """
        return self.__response(
            self.POST(
                f'v1/customers/{id_customer}/cards',
                {
                    'token':token,
                    'payment_method_id':'visa',
                    'issuer_id':25
                }
            )
        )

if __name__ == '__main__':
    mp = MercadoPago()
    payment_list = []

    # Dados do cliente.
    client_id = '150430708-5j4UDdrWJgXk7R'  # ID de cliente na MercadoPago.
    client_mail = 'uriel2@gmail.com'
    client_name_last = 'Alves'
    client_name_first = 'Uriel'
    client_token_card = 'a05a7880946893e4ec5c07e11780c551'
    client_card = Card(

        card_number='4001000200030005',
        security_code='123',
        expiration_month='12',
        expiration_year='2023',
        cardholder={
            'name':'Uriel Alves da Silva',
            'identification':{
                'CPF':'11791724426'
            }
        }
    )

    #identification_types = mp.GET_identification_types()
    payment_methods = mp.GET_payment_methods()

    # Teste gerar cartão
    if True:
        mp.POST_card(client_card)

    if True:
        # Teste registrar cliente.
        try:
            payment_customer = mp.POST_customer(
                email=client_mail,
                first_name=client_name_first,
                last_name=client_name_last
            )
        except exceptions_mercadopago.CustomerAlreadyExistsException:
            pass

    if True:
        # Teste buscar cliente por EMAIL.
        mp.GET_customer_search(
            email=client_mail
        )

    if True:
        # Teste buscar cliente por ID.
        mp.GET_customer(
            id=client_id
        )

    if False:
        # Teste atrelar cartão a um cliente.
        try:
            print('SOU EU!')
            mp.POST_customer_card(
                id_customer=client_id,
                token=client_token_card
            )
        except exceptions_mercadopago.CardDoesNotExistsException:
            pass


