import logging
from sys import argv

from asc.core.ABC import ControlTool
from asc.core.tools.marketplace.mercadopago import data
from asc.core.tools.marketplace.mercadopago.api import MercadoPago
from asc.core.filters import Crypme
from asc.models import PaymentMethod
from asc.models import User as MercadoPagoUser, Card
from asc.models.marketplace import User as MarketPlaceUser, UserCard
from asc.exceptions import marketplace as exceptions_marketplace, \
    mercadopago as exceptions_mercadopago

logging.basicConfig(level=logging.INFO)
"""
    Para utilização/renovação de token melhor envio:
        1 - obter o URL e enviar para o cliente
        2 - postar oauth2_code para registrar autorização
        3 - aplicar refresh token para atualizar oauth2_code
"""
ID_ORDER_DELIVERY = 187

cryp = Crypme()

class ControlToolHandler(ControlTool):

    def __init__(self):
        super(ControlToolHandler, self).__init__()

        self.__MP = MercadoPago()

    def save_payment_methods(self) -> bool:
        """ Busca e registra métodos de pagamentos aceitos.
        :rtype: object
        :return: bool
        """
        payment_methods = self.__MP.GET_payment_methods()
        # Encontra código em cache
        for method in payment_methods:
            model = PaymentMethod()
            model.id = method.get('id')
            model.name = method.get('name')
            model.id_payment_type = method.get('payment_type_id')
            model.active = bool(method.get('status') == 'active')
            self.session.add(model)

        self.commit()
        return True

    def get_client_id(self, id_user:int) -> int:
        """ Registra ou obtém mercadopago_user.id_customer.
        :return: ID do cliente registrado no mercado pago.
        """
        user = self.session \
            .query(
                MarketPlaceUser.name,
                MarketPlaceUser.mail,
                MercadoPagoUser.id_client
            ) \
            .join(MercadoPagoUser,
                  MercadoPagoUser.id_user
                  == MarketPlaceUser.id_user, isouter=True) \
            .where(MarketPlaceUser.id_user == id_user) \
            .first()

        if not user:
            raise exceptions_marketplace.UserDoesNotExists(id_user)

        customer_id = user.id_client
        customer_name = user.name.split(' ')

        if not customer_id:
            try:
                customer_data = self.__MP.POST_customer(
                    email=user.mail,
                    first_name=customer_name[0],
                    last_name=customer_name[-1]
                )

                customer = MercadoPagoUser()
                customer.id = id_user
                customer.id_customer = customer_data['id']
                customer.id_client = customer_data['client_id']
                customer.id_user = customer_data['user_id']
                self.session.add(customer)
                customer_id = customer.id_customer

            except (exceptions_mercadopago \
                    .CustomerAlreadyExistsException) as e:
                logging.error(e)
                customer = self.__MP.GET_customer_search(user.mail)
                customer_id = customer['results'][0]['id']

        return customer_id

    def generate_card_token(self, id_user_card:int):
        """
        Registra cartão na mercadopago.
        :param id_user_card:
        :return:
        """

        card = self.session \
            .query(
                UserCard.nr_card,
                UserCard.nr_cvv,
                UserCard.dt_add,
                UserCard.dt_exp,
                MarketPlaceUser.name,
                MarketPlaceUser.udoc) \
            .join(MarketPlaceUser,
                  MarketPlaceUser.id_user
                  == UserCard.id_user) \
            .where(UserCard.id_user_card
                   ==id_user_card) \
            .first()

        cryp.getGrid(card.dt_add)
        card_holder = data.CardHolder(
            name=card.name,
            identification=dict(CPF=card.udoc)
        )
        card = data.Card(
            card_number=cryp.btoa(card.nr_card),
            security_code=card.nr_cvv,
            expiration_month=card.dt_exp[:2],
            expiration_year=f'20{card.dt_exp[2:]}',
            cardholder=card_holder
        )

        POST_card = self.__MP.POST_card(card)

        if not 'id' in POST_card:
            raise Exception('ID não encontrado em POST_card')

        return POST_card

    def apply_card_token(self, card_token:str, id_user:int, id_user_card:int):
        """
        Renova ou cria card_token do usuário.
        :param card_token:
        :param id_user: user.id_user
        :param id_user: user_card.id_user_card
        :return:
        """

        customer_card = self.__MP.POST_customer_card(
            id_customer=self.get_client_id(id_user),
            token=card_token
        )

        mercadopago_card = self.session \
            .query(Card) \
            .where(Card.id_user_card == id_user_card) \
            .scalar()

        if not mercadopago_card:
            mercadopago_card = Card()

        mercadopago_card.secret = card_token
        mercadopago_card.id_payment_method = customer_card \
            .get('payment_method')['id']

        if not mercadopago_card.id_user_card:
            # Quando ainda não existir relação, INSERT.
            mercadopago_card.id_user_card = id_user_card
            self.session.add(mercadopago_card)

    def __authorize_card(self, id_user:int, id_user_card:int):
        """
        Atualiza token do cartão para utiliza-lo em compras.
        :param id_user:
        :param id_user_card:
        :return:
        """
        card_token = self.generate_card_token(
            id_user_card=id_user_card
        )
        card_customer = self.apply_card_token(
            id_user=id_user,
            id_user_card=id_user_card,
            card_token=card_token['id']
        )
        return card_token

    def __mount_payment_pix(self):
        return data.PaymentPix()

    def __mount_payment_card(self, id_user_card:int):
        customer_card = self.session \
            .query(
                Card.secret,
                Card.id_payment_method) \
            .where(
                Card.id_user_card == id_user_card) \
            .join(
                PaymentMethod,
                PaymentMethod.id == Card.id_payment_method) \
            .first()

        return data.PaymentCard(
            token=customer_card.secret,
            issuer_id=25,
            payment_method_id=customer_card.id_payment_method,
        )

    def send_payment(self, id_user:int, amount:int, **props):
        """
        Solicita pagamento com a mercadopago.
        :param id_user: usuário responsável.
        :param amount: valor da transação.
        :param id_user_card:
        :return:
        """
        id_user_card:int = props.get('id_user_card')

        customer_user = self.session \
            .query(
                MarketPlaceUser.name,
                MarketPlaceUser.udoc,
                MarketPlaceUser.utel,
                MarketPlaceUser.mail,
                MercadoPagoUser.id_customer) \
            .join(
                MercadoPagoUser,
                MercadoPagoUser.id
                == MarketPlaceUser.id_user) \
            .where(MarketPlaceUser.id_user == id_user) \
            .first()

        if not customer_user:
            raise exceptions_marketplace.UserDoesNotExists(id_user)

        customer_user = customer_user._asdict()

        if id_user_card:
            self.__authorize_card(
                id_user=id_user,
                id_user_card=id_user_card
            )
            payment = self.__mount_payment_card(prop_id_user_card)
        else:
            payment = self.__mount_payment_pix()

        payment.transaction_amount = amount
        payment.payer.email = customer_user['mail']
        payment.payer.mount_name(customer_user['name'])
        payment.payer.mount_identification(customer_user['udoc'])
        payment.order.id = 1

        return self.__MP.POST_payment(
            payment
        )


if __name__ == '__main__':
    #ControlToolHandler().save_payment_methods()

    #TODO - 1: Quando usuário já existir, precisamos seguir adiante.
    #RAISE Não funciona, por algum motivo.

    try:

        cth = ControlToolHandler()

        id_user = 1
        #id_user_card = 96
        id_user_card = None

        cth.send_payment(
            amount=100,
            id_user=id_user,
            id_user_card=id_user_card,
        )
        cth.commit()

    except exceptions_mercadopago.CustomerAlreadyExistsException:
        pass
    except Exception as e:
        print(type(e))
        print(f'EXCEPTION --> {e}')

